## This program is free software; you can redistribute it and/or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation; either version 3 of the License, or (at your option) any later
## version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, see <http://www.gnu.org/licenses/>.
##
## Tzvetomir Tzvetanov, 2023/03/09
##
function ANOVA_WS = nANOVA_WS( In_data, TextOut )
%
%     function ANOVA_WS = nANOVA_WS( In_data, TextOut )
%
%   Computes the n way within subjects ANOVA (Univariate approach,
%   e.g. book of Maxwell and Delaney, "Designing experiments and
%   analyzing data", second edition, ch. 12, 2004) with epsilon-hat
%   Geisser-Greenhouse adjustment and epsilon-tilde Huynh-Feldt
%   adjustment. If TextOut is 'yes', then it prints out an AOV table
%   of the results.
%
%   For the analysis, the input matrix should contain the subjects as
%   last dimension of the matrix. Output, a structure containing the
%   ANOVA table. For the moment n>=2 and n<=5.
%
% Tzvetomir Tzvetanov, 20-01-2015
% Vis Res Lab, USTC, Hefei, P.R. China
%
% tt 20150122 trying a general n-way anova code, hard-wired
%   programming, ie spagheti-type of code.... not nice but works.
%
% further infos; for the epsilon_hat and tilde adjustment
% the subfunction "[M,MList] = GenOrthogComps(nTreats,NormFlag)" was
% found on internet and, as written in its "Help", it is
% written on 090131 by Matthew Nelson

if nargin < 1, help nANOVA_WS; ANOVA_WS = []; return; end
if nargin < 2, TextOut = 'no'; end
ND = numel( size( In_data ) ); % dimension of the data and anova
if ND-1 > 5
  fprintf('\n  --> TOO BIG ANOVA -- LIMITED TO 5-way...');
  help nANOVA_WS; return;
end

% fprintf('\n  --> You are making a %d-way within subject ANOVA !',ND-1);
ndim = size( In_data );

% means
mu = mean( In_data(:) ); % mean of data
%
for ii = 1:ND % last dimension is subjects
  %
  % main effects
  kk = 1:ND; kk(ii) = [];
  a = mean( In_data, kk(1) );
  for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
  A_to_S{ii} = reshape( a, [ 1, ndim(ii) ] );
  if ii < ND
    % AS, BS, ... ZS
    kk = 1:ND-1; kk(ii) = [];
    a = mean( In_data, kk(1) );
    for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
    AS_to_ZS{ii} = reshape( a, [ ndim(ii) ndim(end) ]);
    %
    % covariance matrices
    cov_A_to_Z{ii} = cov( AS_to_ZS{ii}' ); % covariance matrices
    % epsilon-hat values
    b = ndim(ii);
    e_hat_A_to_Z(ii) = b * b / ( b - 1 ) * ...
      ( mean( diag( cov_A_to_Z{ii} ) ) - mean( cov_A_to_Z{ii}(:) ) ).^2 / ...
      ( sum( cov_A_to_Z{ii}(:).^2 ) - 2 * b * sum( ( mean( cov_A_to_Z{ii}, 2 ) ).^2 ) + ...
      b * b * ( mean( cov_A_to_Z{ii}(:) ) ).^2 );
    % epsilon-tilde values (Huynh-Feldt)
    etilde_A_to_Z(ii) =  ( ndim(end) * ( b - 1 ) * e_hat_A_to_Z(ii) - 2 ) / ...
      ( b - 1 ) / ( ndim(end) - 1 - ( b - 1 ) * e_hat_A_to_Z(ii) );
    if etilde_A_to_Z(ii)>1, etilde_A_to_Z(ii)=1; end % cannot be above one !
    %
    % second order effects
    i1 = ii;
    for i2 = (i1+1):ND-1
      % AB, AC, AD, ...
      kk = 1:ND; kk([i1 i2]) = []; a = mean( In_data, kk(1) );
      for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
      AB_to_YZ{i1,i2} = reshape( a, [ ndim(i1) ndim(i2) ] );
      % ABS, ACS, ADS, ...
      if ND-1 > 2
        kk = 1:ND-1; kk([i1 i2]) = []; a = mean( In_data, kk(1) );
        for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
        ABS_to_YZS{i1,i2} = reshape( a, [ ndim(i1) ndim(i2) ndim(end) ]);
      else
        ABS_to_YZS{i1,i2} = In_data;
      end
      %
      % covariance matrices
      M = GenOrthogComps( [ ndim(i1), ndim(i2) ] ); aa = [];
      for ia = 1:ndim(i1)
        for ib = 1:ndim(i2)
          for in =1:ndim(end)
            aa(in,(ia-1)*ndim(i2)+ib) = ABS_to_YZS{i1,i2}(ia,ib,in);
          end
        end
      end
      cov_AB_to_YZ{i1,i2} = cov( aa * M{3}' );
      b = ( ndim(i1) - 1 ) * ( ndim(i2) - 1 );
      e_hat_AB_to_YZ(i1,i2) = ...
        sum( diag( cov_AB_to_YZ{i1,i2} ) )^2 / ...
        b / sum( cov_AB_to_YZ{i1,i2}(:).^2 );
      % epsilon-tilde values (Huynh-Feldt)
      etilde_AB_to_YZ(i1,i2) =  ( ndim(end) * b * e_hat_AB_to_YZ(i1,i2) - 2 ) / ...
        b / ( ndim(end) - 1 - b * e_hat_AB_to_YZ(i1,i2) );
      if etilde_AB_to_YZ(i1,i2)>1, etilde_AB_to_YZ(i1,i2)=1; end % cannot be above one !
      %
      % third order effects
      if ND-1 >=3 % % last dim is subjects...
        for i3 = (i2+1):ND-1
          % ABC, ABD, ABE, ...
          kk = 1:ND; kk([i1 i2 i3]) = []; a = mean( In_data, kk(1) );
          for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
          ABC_to_XYZ{i1,i2,i3} = reshape( a, [ ndim(i1) ndim(i2) ndim(i3) ] );
          % ABCS, ABDS, ABES, ...
          if ND-1 > 3 % if more than 3 factors, sumirrelevant factors
            kk = 1:ND-1; kk([i1 i2 i3]) = []; a = mean( In_data, kk(1) );
            for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
            ABCS_to_XYZS{i1,i2,i3} = reshape( a, [ ndim(i1) ndim(i2) ndim(i3) ndim(end) ]);
          else
            ABCS_to_XYZS{i1,i2,i3} = In_data;
          end
          %
          % covariance matrices
          M = GenOrthogComps( [ ndim(i1), ndim(i2), ndim(i3) ] ); aa = [];
          for ia = 1:ndim(i1)
            for ib = 1:ndim(i2)
              for ic = 1:ndim(i3)
                for in =1:ndim(end)
                  aa(in,(ia-1)*ndim(i2)*ndim(i3)+(ib-1)*ndim(i3)+ic) = ...
                    ABCS_to_XYZS{i1,i2,i3}(ia,ib,ic,in);
                end
              end
            end
          end
          cov_ABC_to_XYZ{i1,i2,i3} = cov( aa * M{end}' );
          b = ( ndim(i1) - 1 ) * ( ndim(i2) - 1 ) * ( ndim(i3) -1 );
          e_hat_ABC_to_XYZ(i1,i2,i3) = ...
            sum( diag( cov_ABC_to_XYZ{i1,i2,i3} ) )^2 / ...
            b / sum( cov_ABC_to_XYZ{i1,i2,i3}(:).^2 );
          % epsilon-tilde values (Huynh-Feldt)
          etilde_ABC_to_XYZ(i1,i2,i3) = ...
            ( ndim(end) * b * e_hat_ABC_to_XYZ(i1,i2,i3) - 2 ) / ...
            b / ( ndim(end) - 1 - b * e_hat_ABC_to_XYZ(i1,i2,i3) );
          if etilde_ABC_to_XYZ(i1,i2,i3)>1, etilde_ABC_to_XYZ(i1,i2,i3)=1; end % cannot be above one !
          %
          % fourth order effects
          if ND-1 >= 4 % % last dim is subjects...
            for i4 = (i3+1):ND-1
              % ABCD, ABCE, ABCF, ...
              kk = 1:ND; kk([i1 i2 i3 i4]) = []; a = mean( In_data, kk(1) );
              for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
              ABCD_to_WXYZ{i1,i2,i3,i4} = reshape( a, [ ndim(i1) ndim(i2) ndim(i3) ndim(i4) ] );
              % ABCDS ...
              if ND-1 > 4 % if more than 4 factors, sumirrelevant factors
                kk = 1:ND-1; kk([i1 i2 i3 i4]) = []; a = mean( In_data, kk(1) );
                for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
                ABCDS_to_WXYZS{i1,i2,i3,i4} = reshape( a, [ ndim(i1) ndim(i2) ndim(i3) ndim(i4) ndim(end) ]);
              else
                ABCDS_to_WXYZS{i1,i2,i3,i4} = In_data;
              end
              %
              % covariance matrices
              M = GenOrthogComps( [ ndim(i1), ndim(i2), ndim(i3), ndim(i4) ] ); aa = [];
              for ia = 1:ndim(i1)
                for ib = 1:ndim(i2)
                  for ic = 1:ndim(i3)
                    for id = 1:ndim(i4)
                      for in =1:ndim(end)
                        aa(in, (ia-1)*ndim(i2)*ndim(i3)*ndim(i4)+(ib-1)*ndim(i3)*ndim(i4)+(ic-1)*ndim(i4)+id ) = ...
                          ABCDS_to_WXYZS{i1,i2,i3,i4}(ia,ib,ic,id,in);
                      end
                    end
                  end
                end
              end
              cov_ABCD_to_WXYZ{i1,i2,i3,i4} = cov( aa * M{end}' );
              b = ( ndim(i1) - 1 )*( ndim(i2) - 1 )*( ndim(i3) -1 )*( ndim(i4) - 1 );
              e_hat_ABCD_to_WXYZ(i1,i2,i3,i4) = ...
                sum( diag( cov_ABCD_to_WXYZ{i1,i2,i3,i4} ) )^2 / ...
                b / sum( cov_ABCD_to_WXYZ{i1,i2,i3,i4}(:).^2 );
              % epsilon-tilde values (Huynh-Feldt)
              etilde_ABCD_to_WXYZ(i1,i2,i3,i4) = ...
                ( ndim(end) * b * e_hat_ABCD_to_WXYZ(i1,i2,i3,i4) - 2 ) / ...
                b / ( ndim(end) - 1 - b * e_hat_ABCD_to_WXYZ(i1,i2,i3,i4) );
              if etilde_ABCD_to_WXYZ(i1,i2,i3,i4)>1, etilde_ABCD_to_WXYZ(i1,i2,i3,i4)=1; end % cannot be above one !
              %
              % fifth order effects
              if ND-1 >= 5 % % last dim is subjects...
                for i5 = (i4+1):ND-1
                  % ABCDE, ...
                  kk = 1:ND; kk([i1 i2 i3 i4 i5]) = []; a = mean( In_data, kk(1) );
                  for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
                  ABCDE_to_VWXYZ{i1,i2,i3,i4,i5} = reshape( a, ...
                    [ ndim(i1) ndim(i2) ndim(i3) ndim(i4) ndim(i5) ] );
                  % ABCDES ...
                  if ND-1 > 5 % if more than 5 factors, sumirrelevant factors
                    kk = 1:ND-1; kk([i1 i2 i3 i4 i5]) = []; a = mean( In_data, kk(1) );
                    for jj = 2:numel(kk), a = mean( a, kk(jj) ); end
                    ABCDES_to_VWXYZS{i1,i2,i3,i4,i5} = reshape( a, ...
                      [ ndim(i1) ndim(i2) ndim(i3) ndim(i4) ndim(i5) ndim(end) ]);
                  else
                    ABCDES_to_VWXYZS{i1,i2,i3,i4,i5} = In_data;
                  end
                  %
                  % covariance matrices
                  M = GenOrthogComps( [ ndim(i1), ndim(i2), ndim(i3), ndim(i4), ndim(i5) ] ); aa = [];
                  for ia = 1:ndim(i1)
                    for ib = 1:ndim(i2)
                      for ic = 1:ndim(i3)
                        for id = 1:ndim(i4)
                          for ie = 1:ndim(i5)
                            for in =1:ndim(end)
                              aa(in, (ia-1)*ndim(i2)*ndim(i3)*ndim(i4)*ndim(i5)+(ib-1)*ndim(i3)*ndim(i4)*ndim(i5)+(ic-1)*ndim(i4)*ndim(i5)+(id-1)*ndim(i5)+ie ) = ...
                                ABCDES_to_VWXYZS{i1,i2,i3,i4,i5}(ia,ib,ic,id,ie,in);
                            end
                          end
                        end
                      end
                    end
                  end
                  cov_ABCDE_to_VWXYZ{i1,i2,i3,i4,i5} = cov( aa * M{end}' );
                  b = ( ndim(i1) - 1 )*( ndim(i2) - 1 )*( ndim(i3) -1 )*( ndim(i4) - 1 )*( ndim(i5) - 1 );
                  e_hat_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) = ...
                    sum( diag( cov_ABCDE_to_VWXYZ{i1,i2,i3,i4,i5} ) )^2 / ...
                    b / sum( cov_ABCDE_to_VWXYZ{i1,i2,i3,i4,i5}(:).^2 );
                  % epsilon-tilde values (Huynh-Feldt)
                  etilde_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) = ...
                    ( ndim(end) * b * e_hat_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) - 2 ) / ...
                    b / ( ndim(end) - 1 - b * e_hat_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) );
                  if etilde_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5)>1, etilde_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5)=1; end % cannot be above one !
                  %
                end
              end
            end
          end
        end
      end
    end
  end
end

% sum of squares
for ii = 1:ND-1
  %
  % sum of squares
%   sum( A_to_S{ii} )
%   sum( ( A_to_S{ii} - mu ).^2 )
  A_to_Z_ss(ii,1:2) = ...
    [ prod(ndim)/ndim(ii) * sum( ( A_to_S{ii} - mu ).^2 ), ndim(ii)-1 ];
  AS_to_ZS_ss(ii,1:2) = ...
    [ prod(ndim)/ndim(ii)/ndim(end) * ...
    sum( sum( ( AS_to_ZS{ii} - repmat(A_to_S{ND}, [ ndim(ii) 1 ] ) - ...
                repmat(A_to_S{ii}',[ 1 ndim(end) ] ) + mu ).^2 ) ), ...
      (ndim(ii)-1)*(ndim(end)-1) ];
  %
  % F and p-values
  F_A_to_Z(ii) = ...
    A_to_Z_ss(ii,1) / A_to_Z_ss(ii,2) / ...
    ( AS_to_ZS_ss(ii,1) / AS_to_ZS_ss(ii,2) );
  p_A_to_Z(ii) = 1 - fcdf( F_A_to_Z(ii), A_to_Z_ss(ii,2), AS_to_ZS_ss(ii,2) );
  p_A_to_Z_GG(ii) = 1 - fcdf( F_A_to_Z(ii), ...
    e_hat_A_to_Z(ii)*A_to_Z_ss(ii,2), e_hat_A_to_Z(ii)*AS_to_ZS_ss(ii,2) );
  p_A_to_Z_HF(ii) = 1 - fcdf( F_A_to_Z(ii), ...
    etilde_A_to_Z(ii)*A_to_Z_ss(ii,2), etilde_A_to_Z(ii)*AS_to_ZS_ss(ii,2) );
  %
  % second order SS
  i1 = ii;
  for i2 = (i1+1):ND-1
    AB_to_YZ_ss{i1,i2} = [ prod(ndim) / ndim(i1) / ndim(i2) * sum( sum( ...
      ( AB_to_YZ{i1,i2} - repmat(A_to_S{i1}',[ 1 ndim(i2) ]) - ...
        repmat(A_to_S{i2},[ ndim(i1) 1 ])+mu).^2 ) ), ...
      (ndim(i1)-1)*(ndim(i2)-1) ] ;
    %
    ABS_to_YZS_ss{i1,i2}(1:2) = [ 0 (ndim(i1)-1)*(ndim(i2)-1)*(ndim(end)-1) ];
    for ia = 1:ndim(i1)
      for ib = 1:ndim(i2)
        for in = 1:ndim(end)
          ABS_to_YZS_ss{i1,i2}(1) = ABS_to_YZS_ss{i1,i2}(1) + ...
            prod(ndim) / ndim(i1) / ndim(i2) / ndim(end) * ...
            ( ABS_to_YZS{i1,i2}(ia,ib,in) ...
            - AS_to_ZS{i1}(ia,in) - AS_to_ZS{i2}(ib,in) - AB_to_YZ{i1,i2}(ia,ib) ...
            + A_to_S{end}(in) + A_to_S{i1}(ia) + A_to_S{i2}(ib) - mu ).^2;
        end
      end
    end
    %
    % F and p-values
    F_AB_to_YZ(i1,i2) = AB_to_YZ_ss{i1,i2}(1) / AB_to_YZ_ss{i1,i2}(2) / ...
      ( ABS_to_YZS_ss{i1,i2}(1) / ABS_to_YZS_ss{i1,i2}(2) );
    p_AB_to_YZ(i1,i2) = 1 - fcdf( ...
      F_AB_to_YZ(i1,i2), AB_to_YZ_ss{i1,i2}(2), ABS_to_YZS_ss{i1,i2}(2) );
    p_AB_to_YZ_GG(i1,i2) = 1 - fcdf( F_AB_to_YZ(i1,i2), ...
      e_hat_AB_to_YZ(i1,i2) * AB_to_YZ_ss{i1,i2}(2), ...
      e_hat_AB_to_YZ(i1,i2) * ABS_to_YZS_ss{i1,i2}(2) );
    p_AB_to_YZ_HF(i1,i2) = 1 - fcdf( F_AB_to_YZ(i1,i2), ...
      etilde_AB_to_YZ(i1,i2) * AB_to_YZ_ss{i1,i2}(2), ...
      etilde_AB_to_YZ(i1,i2) * ABS_to_YZS_ss{i1,i2}(2) );
    %
    if ND-1 >= 3 % % last dim is subjects...
      % third order effects
      for i3 = (i2+1):ND-1
        %
        ABC_to_XYZ_ss{i1,i2,i3}(1:2) = [ 0 (ndim(i1)-1)*(ndim(i2)-1)*(ndim(i3)-1) ];
        for ia = 1:ndim(i1)
          for ib = 1:ndim(i2)
            for ic = 1:ndim(i3)
              ABC_to_XYZ_ss{i1,i2,i3}(1) = ABC_to_XYZ_ss{i1,i2,i3}(1) + ...
                prod(ndim) / ndim(i1) / ndim(i2) / ndim(i3) * ...
                ( ABC_to_XYZ{i1,i2,i3}(ia,ib,ic) ...
                - AB_to_YZ{i1,i2}(ia,ib) ...
                - AB_to_YZ{i1,i3}(ia,ic) ...
                - AB_to_YZ{i2,i3}(ib,ic) ...
                + A_to_S{i1}(ia) ...
                + A_to_S{i2}(ib) ...
                + A_to_S{i3}(ic) ...
                - mu ).^2;
            end
          end
        end
        %
        ABCS_to_XYZS_ss{i1,i2,i3}(1:2) = [ 0 (ndim(i1)-1)*(ndim(i2)-1)*(ndim(i3)-1)*(ndim(end)-1) ];
        for ia = 1:ndim(i1)
          for ib = 1:ndim(i2)
            for ic = 1:ndim(i3)
              for in = 1:ndim(end)
                ABCS_to_XYZS_ss{i1,i2,i3}(1) = ABCS_to_XYZS_ss{i1,i2,i3}(1) + ...
                  prod(ndim) / ndim(i1) / ndim(i2) / ndim(i3) / ndim(end) * ...
                  ( ABCS_to_XYZS{i1,i2,i3}(ia,ib,ic,in) ...
                  - ABC_to_XYZ{i1,i2,i3}(ia,ib,ic) ...
                  - ABS_to_YZS{i2,i3}(ib,ic,in) ...
                  - ABS_to_YZS{i1,i3}(ia,ic,in) ...
                  - ABS_to_YZS{i1,i2}(ia,ib,in) ...
                  + AS_to_ZS{i1}(ia,in) ...
                  + AS_to_ZS{i2}(ib,in) ...
                  + AS_to_ZS{i3}(ic,in) ...
                  + AB_to_YZ{i1,i2}(ia,ib) ...
                  + AB_to_YZ{i1,i3}(ia,ic) ...
                  + AB_to_YZ{i2,i3}(ib,ic) ...
                  - A_to_S{end}(in) ...
                  - A_to_S{i1}(ia) ...
                  - A_to_S{i2}(ib) ...
                  - A_to_S{i3}(ic) ...
                  + mu ).^2;
              end
            end
          end
        end
        %
        % F and p-values
        F_ABC_to_XYZ(i1,i2,i3) = ABC_to_XYZ_ss{i1,i2,i3}(1) / ABC_to_XYZ_ss{i1,i2,i3}(2) / ...
          ( ABCS_to_XYZS_ss{i1,i2,i3}(1) / ABCS_to_XYZS_ss{i1,i2,i3}(2) );
        p_ABC_to_XYZ(i1,i2,i3) = 1 - fcdf( F_ABC_to_XYZ(i1,i2,i3), ...
          ABC_to_XYZ_ss{i1,i2,i3}(2), ABCS_to_XYZS_ss{i1,i2,i3}(2) );
        p_ABC_to_XYZ_GG(i1,i2,i3) = 1 - fcdf( F_ABC_to_XYZ(i1,i2,i3), ...
          e_hat_ABC_to_XYZ(i1,i2,i3) * ABC_to_XYZ_ss{i1,i2,i3}(2), ...
          e_hat_ABC_to_XYZ(i1,i2,i3) * ABCS_to_XYZS_ss{i1,i2,i3}(2) );
        p_ABC_to_XYZ_HF(i1,i2,i3) = 1 - fcdf(  F_ABC_to_XYZ(i1,i2,i3), ...
          etilde_ABC_to_XYZ(i1,i2,i3) * ABC_to_XYZ_ss{i1,i2,i3}(2), ...
          etilde_ABC_to_XYZ(i1,i2,i3) * ABCS_to_XYZS_ss{i1,i2,i3}(2) );
        %
        if ND-1 >= 4
          %
          % fourth order effects
          for i4 = (i3+1):ND-1
            %
            ABCD_to_WXYZ_ss{i1,i2,i3,i4}(1:2) = [ 0 ...
              (ndim(i1)-1)*(ndim(i2)-1)*(ndim(i3)-1)*(ndim(i4)-1) ];
            for ia = 1:ndim(i1)
              for ib = 1:ndim(i2)
                for ic = 1:ndim(i3)
                  for id = 1:ndim(i4)
                    ABCD_to_WXYZ_ss{i1,i2,i3,i4}(1) = ABCD_to_WXYZ_ss{i1,i2,i3,i4}(1) + ...
                      prod(ndim) / ndim(i1) / ndim(i2) / ndim(i3) / ndim(i4) * ...
                      ( ABCD_to_WXYZ{i1,i2,i3,i4}(ia,ib,ic,id) ...
                      - ABC_to_XYZ{i1,i2,i3}(ia,ib,ic) ...
                      - ABC_to_XYZ{i1,i2,i4}(ia,ib,id) ...
                      - ABC_to_XYZ{i1,i3,i4}(ia,ic,id) ...
                      - ABC_to_XYZ{i2,i3,i4}(ib,ic,id) ...
                      + AB_to_YZ{i1,i2}(ia,ib) ...
                      + AB_to_YZ{i1,i3}(ia,ic) ...
                      + AB_to_YZ{i1,i4}(ia,id) ...
                      + AB_to_YZ{i2,i3}(ib,ic) ...
                      + AB_to_YZ{i2,i4}(ib,id) ...
                      + AB_to_YZ{i3,i4}(ic,id) ...
                      - A_to_S{i1}(ia) ...
                      - A_to_S{i2}(ib) ...
                      - A_to_S{i3}(ic) ...
                      - A_to_S{i4}(id) ...
                      + mu ).^2;
                  end
                end
              end
            end
            %
            ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(1:2) = [ 0 ...
              (ndim(i1)-1)*(ndim(i2)-1)*(ndim(i3)-1)*(ndim(i4)-1)*(ndim(end)-1) ];
            for ia = 1:ndim(i1)
              for ib = 1:ndim(i2)
                for ic = 1:ndim(i3)
                  for id = 1:ndim(i4)
                    for in = 1:ndim(end)
                      ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(1) = ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(1) + ...
                        prod(ndim) / ndim(i1) / ndim(i2) / ndim(i3) / ndim(i4) / ndim(end) * ...
                        ( ABCDS_to_WXYZS{i1,i2,i3,i4}(ia,ib,ic,id,in) ...
                        - ABCD_to_WXYZ{i1,i2,i3,i4}(ia,ib,ic,id) ...
                        - ABCS_to_XYZS{i1,i2,i3}(ia,ib,ic,in)  ...
                        - ABCS_to_XYZS{i1,i2,i4}(ia,ib,id,in) ...
                        - ABCS_to_XYZS{i1,i3,i4}(ia,ic,id,in) ...
                        - ABCS_to_XYZS{i2,i3,i4}(ib,ic,id,in) ...
                        + ABC_to_XYZ{i1,i2,i3}(ia,ib,ic) ...
                        + ABC_to_XYZ{i1,i2,i4}(ia,ib,id) ...
                        + ABC_to_XYZ{i1,i3,i4}(ia,ic,id) ...
                        + ABC_to_XYZ{i2,i3,i4}(ib,ic,id) ...
                        + ABS_to_YZS{i1,i2}(ia,ib,in) ...
                        + ABS_to_YZS{i1,i3}(ia,ic,in) ...
                        + ABS_to_YZS{i1,i4}(ia,id,in) ...
                        + ABS_to_YZS{i2,i3}(ib,ic,in) ...
                        + ABS_to_YZS{i2,i4}(ib,id,in) ...
                        + ABS_to_YZS{i3,i4}(ic,id,in) ...
                        - AB_to_YZ{i1,i2}(ia,ib) ...
                        - AB_to_YZ{i1,i3}(ia,ic) ...
                        - AB_to_YZ{i1,i4}(ia,id) ...
                        - AB_to_YZ{i2,i3}(ib,ic) ...
                        - AB_to_YZ{i2,i4}(ib,id) ...
                        - AB_to_YZ{i3,i4}(ic,id) ...
                        - AS_to_ZS{i1}(ia,in) ...
                        - AS_to_ZS{i2}(ib,in) ...
                        - AS_to_ZS{i3}(ic,in) ...
                        - AS_to_ZS{i4}(id,in) ...
                        + A_to_S{end}(in) ...
                        + A_to_S{i1}(ia) ...
                        + A_to_S{i2}(ib) ...
                        + A_to_S{i3}(ic) ...
                        + A_to_S{i4}(id) ...
                        - mu ).^2;
                    end
                  end
                end
              end
            end
            %
            % F and p-values
            F_ABCD_to_WXYZ(i1,i2,i3,i4) = ...
              ABCD_to_WXYZ_ss{i1,i2,i3,i4}(1) / ABCD_to_WXYZ_ss{i1,i2,i3,i4}(2) / ...
              ( ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(1) / ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(2) );
            p_ABCD_to_WXYZ(i1,i2,i3,i4) = 1 - fcdf( F_ABCD_to_WXYZ(i1,i2,i3,i4), ...
              ABCD_to_WXYZ_ss{i1,i2,i3,i4}(2), ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(2) );
            p_ABCD_to_WXYZ_GG(i1,i2,i3,i4) = 1 - fcdf( F_ABCD_to_WXYZ(i1,i2,i3,i4), ...
              e_hat_ABCD_to_WXYZ(i1,i2,i3,i4) * ABCD_to_WXYZ_ss{i1,i2,i3,i4}(2), ...
              e_hat_ABCD_to_WXYZ(i1,i2,i3,i4) * ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(2) );
            p_ABCD_to_WXYZ_HF(i1,i2,i3,i4) = 1 - fcdf(  F_ABCD_to_WXYZ(i1,i2,i3,i4), ...
              etilde_ABCD_to_WXYZ(i1,i2,i3,i4) * ABCD_to_WXYZ_ss{i1,i2,i3,i4}(2), ...
              etilde_ABCD_to_WXYZ(i1,i2,i3,i4) * ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(2) );
            %
            if ND-1 >= 5
              %
              % fifth order effects
              for i5 = (i4+1):ND-1
                %
                ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(1:2) = [ 0 ...
                  (ndim(i1)-1)*(ndim(i2)-1)*(ndim(i3)-1)*(ndim(i4)-1)*(ndim(i5)-1) ];
                for ia = 1:ndim(i1)
                  for ib = 1:ndim(i2)
                    for ic = 1:ndim(i3)
                      for id = 1:ndim(i4)
                        for ie = 1:ndim(i5)
                          ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(1) = ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(1) + ...
                            prod(ndim) / ndim(i1) / ndim(i2) / ndim(i3) / ndim(i4) / ndim(i5) * ...
                            ( ABCDE_to_VWXYZ{i1,i2,i3,i4,i5}(ia,ib,ic,id,ie) ...
                            - ABCD_to_WXYZ{i1,i2,i3,i4}(ia,ib,ic,id) ...
                            - ABCD_to_WXYZ{i1,i2,i3,i5}(ia,ib,ic,ie) ... 
                            - ABCD_to_WXYZ{i1,i2,i4,i5}(ia,ib,id,ie) ...
                            - ABCD_to_WXYZ{i1,i3,i4,i5}(ia,ic,id,ie) ...
                            - ABCD_to_WXYZ{i2,i3,i4,i5}(ib,ic,id,ie) ...
                            + ABC_to_XYZ{i1,i2,i3}(ia,ib,ic) ...
                            + ABC_to_XYZ{i1,i2,i4}(ia,ib,id) ...
                            + ABC_to_XYZ{i1,i2,i5}(ia,ib,ie) ...
                            + ABC_to_XYZ{i1,i3,i4}(ia,ic,id) ...
                            + ABC_to_XYZ{i1,i3,i5}(ia,ic,ie) ...
                            + ABC_to_XYZ{i1,i4,i5}(ia,id,ie) ...
                            + ABC_to_XYZ{i2,i3,i4}(ib,ic,id) ...
                            + ABC_to_XYZ{i2,i3,i5}(ib,ic,ie) ...
                            + ABC_to_XYZ{i2,i4,i5}(ib,id,ie) ...
                            + ABC_to_XYZ{i3,i4,i5}(ic,id,ie) ...
                            - AB_to_YZ{i1,i2}(ia,ib) ...
                            - AB_to_YZ{i1,i3}(ia,ic) ...
                            - AB_to_YZ{i1,i4}(ia,id) ...
                            - AB_to_YZ{i1,i5}(ia,ie) ...
                            - AB_to_YZ{i2,i3}(ib,ic) ...
                            - AB_to_YZ{i2,i4}(ib,id) ...
                            - AB_to_YZ{i2,i5}(ib,ie) ...
                            - AB_to_YZ{i3,i4}(ic,id) ...
                            - AB_to_YZ{i3,i5}(ic,ie) ...
                            - AB_to_YZ{i4,i5}(id,ie) ...
                            + A_to_S{i1}(ia) ...
                            + A_to_S{i2}(ib) ...
                            + A_to_S{i3}(ic) ...
                            + A_to_S{i4}(id) ...
                            + A_to_S{i5}(ie) ...
                            - mu ).^2;
                        end
                      end
                    end
                  end
                end
                %
                ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(1:2) = [ 0 ...
                  (ndim(i1)-1)*(ndim(i2)-1)*(ndim(i3)-1)*(ndim(i4)-1)*(ndim(i5)-1)*(ndim(end)-1) ];
                for ia = 1:ndim(i1)
                  for ib = 1:ndim(i2)
                    for ic = 1:ndim(i3)
                      for id = 1:ndim(i4)
                        for ie = 1:ndim(i5)
                          for in = 1:ndim(end)
                            ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(1) = ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(1) + ...
                              prod(ndim) / ndim(i1) / ndim(i2) / ndim(i3) / ndim(i4) / ndim(i5) / ndim(end) * ...
                              ( ABCDES_to_VWXYZS{i1,i2,i3,i4,i5}(ia,ib,ic,id,ie,in) ...
                              - ABCDE_to_VWXYZ{i1,i2,i3,i4,i5}(ia,ib,ic,id,ie) ...
                              - ABCDS_to_WXYZS{i1,i2,i3,i4}(ia,ib,ic,id,in) ...
                              - ABCDS_to_WXYZS{i1,i2,i3,i5}(ia,ib,ic,ie,in) ...
                              - ABCDS_to_WXYZS{i1,i2,i4,i5}(ia,ib,id,ie,in) ...
                              - ABCDS_to_WXYZS{i1,i3,i4,i5}(ia,ic,id,ie,in) ...
                              - ABCDS_to_WXYZS{i2,i3,i4,i5}(ib,ic,id,ie,in) ...
                              + ABCD_to_WXYZ{i1,i2,i3,i4}(ia,ib,ic,id) ...
                              + ABCD_to_WXYZ{i1,i2,i3,i5}(ia,ib,ic,ie) ...
                              + ABCD_to_WXYZ{i1,i2,i4,i5}(ia,ib,id,ie) ...
                              + ABCD_to_WXYZ{i1,i3,i4,i5}(ia,ic,id,ie) ...
                              + ABCD_to_WXYZ{i2,i3,i4,i5}(ib,ic,id,ie) ...
                              + ABCS_to_XYZS{i1,i2,i3}(ia,ib,ic,in) ...
                              + ABCS_to_XYZS{i1,i2,i4}(ia,ib,id,in) ...
                              + ABCS_to_XYZS{i1,i2,i5}(ia,ib,ie,in) ...
                              + ABCS_to_XYZS{i1,i3,i4}(ia,ic,id,in) ...
                              + ABCS_to_XYZS{i1,i3,i5}(ia,ic,ie,in) ...
                              + ABCS_to_XYZS{i1,i4,i5}(ia,id,ie,in) ...
                              + ABCS_to_XYZS{i2,i3,i4}(ib,ic,id,in) ...
                              + ABCS_to_XYZS{i2,i3,i5}(ib,ic,ie,in) ...
                              + ABCS_to_XYZS{i2,i4,i5}(ib,id,ie,in) ...
                              + ABCS_to_XYZS{i3,i4,i5}(ic,id,ie,in) ...
                              - ABC_to_XYZ{i1,i2,i3}(ia,ib,ic) ...
                              - ABC_to_XYZ{i1,i2,i4}(ia,ib,id) ...
                              - ABC_to_XYZ{i1,i2,i5}(ia,ib,ie) ...
                              - ABC_to_XYZ{i1,i3,i4}(ia,ic,id) ...
                              - ABC_to_XYZ{i1,i3,i5}(ia,ic,ie) ...
                              - ABC_to_XYZ{i1,i4,i5}(ia,id,ie) ...
                              - ABC_to_XYZ{i2,i3,i4}(ib,ic,id) ...
                              - ABC_to_XYZ{i2,i3,i5}(ib,ic,ie) ...
                              - ABC_to_XYZ{i2,i4,i5}(ib,id,ie) ...
                              - ABC_to_XYZ{i3,i4,i5}(ic,id,ie) ...
                              - ABS_to_YZS{i1,i2}(ia,ib,in) ...
                              - ABS_to_YZS{i1,i3}(ia,ic,in) ...
                              - ABS_to_YZS{i1,i4}(ia,id,in) ...
                              - ABS_to_YZS{i1,i5}(ia,ie,in) ...
                              - ABS_to_YZS{i2,i3}(ib,ic,in) ...
                              - ABS_to_YZS{i2,i4}(ib,id,in) ...
                              - ABS_to_YZS{i2,i5}(ib,ie,in) ...
                              - ABS_to_YZS{i3,i4}(ic,id,in) ...
                              - ABS_to_YZS{i3,i5}(ic,ie,in) ...
                              - ABS_to_YZS{i4,i5}(id,ie,in) ...
                              + AB_to_YZ{i1,i2}(ia,ib) ...
                              + AB_to_YZ{i1,i3}(ia,ic) ...
                              + AB_to_YZ{i1,i4}(ia,id) ...
                              + AB_to_YZ{i1,i5}(ia,ie) ...
                              + AB_to_YZ{i2,i3}(ib,ic) ...
                              + AB_to_YZ{i2,i4}(ib,id) ...
                              + AB_to_YZ{i2,i5}(ib,ie) ...
                              + AB_to_YZ{i3,i4}(ic,id) ...
                              + AB_to_YZ{i3,i5}(ic,ie) ...
                              + AB_to_YZ{i4,i5}(id,ie) ...
                              + AS_to_ZS{i1}(ia,in) ...
                              + AS_to_ZS{i2}(ib,in) ...
                              + AS_to_ZS{i3}(ic,in) ...
                              + AS_to_ZS{i4}(id,in) ...
                              + AS_to_ZS{i5}(ie,in) ...
                              - A_to_S{i1}{ia} ...
                              - A_to_S{i2}{ib} ...
                              - A_to_S{i3}{ic} ...
                              - A_to_S{i4}{id} ...
                              - A_to_S{i5}{ie} ...
                              - A_to_S{end}{in} ...
                              + mu ).^2;
                          end
                        end
                      end
                    end
                  end
                end
                %
                % F and p-values
                F_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) = ...
                  ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(1) / ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(2) / ...
                  ( ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(1) / ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(2) );
                p_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) = 1 - fcdf( ...
                  F_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5), ...
                  ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(2), ...
                  ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(2) );
                p_ABCDE_to_VWXYZ_GG(i1,i2,i3,i4,i5) = 1 - fcdf( ...
                  F_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5), ...
                  e_hat_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5)*ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(2), ...
                  e_hat_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5)*ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(2) );
                p_ABCDE_to_VWXYZ_HF(i1,i2,i3,i4,i5) = 1 - fcdf(  ...
                  F_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5), ...
                  etilde_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5)*ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(2), ...
                  etilde_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5)*ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(2) );
                %
              end
            end
          end
        end
      end
    end
  end
end


% saving in the output structure
ANOVA_WS.text = 'SumSquares  df  MS   Fval  p  p(GG)  p(HF) ehat  etilde';
cnt2 = 0; cnt3 = 0; cnt4 = 0; cnt5 = 0;
for ii = 1:ND-1
  ANOVA_WS.AtoZ(ii,:) = ...
    [ A_to_Z_ss(ii,1:2) ...
      A_to_Z_ss(ii,1)/A_to_Z_ss(ii,2) ...
      F_A_to_Z(ii) p_A_to_Z(ii) p_A_to_Z_GG(ii) ...
      p_A_to_Z_HF(ii) e_hat_A_to_Z(ii) etilde_A_to_Z(ii) ];
  % Subject's sumsquares
  ANOVA_WS.AStoZS(ii,:) = [ ...
    AS_to_ZS_ss(ii,:) ...
    AS_to_ZS_ss(ii,1)/AS_to_ZS_ss(ii,2) ];
  i1 = ii;
  for i2 = (ii+1):ND-1
    cnt2 = cnt2 + 1;
    ANOVA_WS.AB(cnt2,:) = [ ...
      AB_to_YZ_ss{i1,i2}(1:2) ...
      AB_to_YZ_ss{i1,i2}(1)/AB_to_YZ_ss{i1,i2}(2)...
      F_AB_to_YZ(i1,i2) ...
      p_AB_to_YZ(i1,i2) ...
      p_AB_to_YZ_GG(i1,i2) ...
      p_AB_to_YZ_HF(i1,i2) ...
      e_hat_AB_to_YZ(i1,i2) ...
      etilde_AB_to_YZ(i1,i2) ];
    ANOVA_WS.ABS(cnt2,:) = [ ...
      ABS_to_YZS_ss{i1,i2} ...
      ABS_to_YZS_ss{i1,i2}(1)/ABS_to_YZS_ss{i1,i2}(2)];
    if ND-1 >= 3
      for i3 = (i2+1):ND-1
        cnt3 = cnt3 + 1;
        ANOVA_WS.ABC(cnt3,:) = [ ...
          ABC_to_XYZ_ss{i1,i2,i3}(1:2) ...
          ABC_to_XYZ_ss{i1,i2,i3}(1)/ABC_to_XYZ_ss{i1,i2,i3}(2)...
          F_ABC_to_XYZ(i1,i2,i3) ...
          p_ABC_to_XYZ(i1,i2,i3) ...
          p_ABC_to_XYZ_GG(i1,i2,i3) ...
          p_ABC_to_XYZ_HF(i1,i2,i3) ...
          e_hat_ABC_to_XYZ(i1,i2,i3) ...
          etilde_ABC_to_XYZ(i1,i2,i3) ];
        ANOVA_WS.ABCS(cnt3,:) = [ ...
          ABCS_to_XYZS_ss{i1,i2,i3} ...
          ABCS_to_XYZS_ss{i1,i2,i3}(1)/ABCS_to_XYZS_ss{i1,i2,i3}(2)];
        if ND-1 >= 4
          for i4 = i3+1:ND-1
            cnt4 = cnt4 + 1;
            ANOVA_WS.ABCD(cnt4,:) = [ ...
              ABCD_to_WXYZ_ss{i1,i2,i3,i4}(1:2) ...
              ABCD_to_WXYZ_ss{i1,i2,i3,i4}(1)/ABCD_to_WXYZ_ss{i1,i2,i3,i4}(2)...
              F_ABCD_to_WXYZ(i1,i2,i3,i4) ...
              p_ABCD_to_WXYZ(i1,i2,i3,i4) ...
              p_ABCD_to_WXYZ_GG(i1,i2,i3,i4) ...
              p_ABCD_to_WXYZ_HF(i1,i2,i3,i4) ...
              e_hat_ABCD_to_WXYZ(i1,i2,i3,i4) ...
              etilde_ABCD_to_WXYZ(i1,i2,i3,i4) ];
            ANOVA_WS.ABCDS(cnt4,:) = [ ...
              ABCDS_to_WXYZS_ss{i1,i2,i3,i4} ...
              ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(1)/ABCDS_to_WXYZS_ss{i1,i2,i3,i4}(2)];
            if ND-1>=5
              for i5 = i4+1:ND-1
                cnt5 = cnt5 + 1;
                ANOVA_WS.ABCDE(cnt5,:) = [ ...
                  ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(1:2) ...
                  ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(1)/ABCDE_to_VWXYZ_ss{i1,i2,i3,i4,i5}(2)...
                  F_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) ...
                  p_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) ...
                  p_ABCDE_to_VWXYZ_GG(i1,i2,i3,i4,i5) ...
                  p_ABCDE_to_VWXYZ_HF(i1,i2,i3,i4,i5) ...
                  e_hat_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) ...
                  etilde_ABCDE_to_VWXYZ(i1,i2,i3,i4,i5) ];
                ANOVA_WS.ABCDES(cnt5,:) = [ ...
                  ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5} ...
                  ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(1)/ABCDES_to_VWXYZS_ss{i1,i2,i3,i4,i5}(2)];
              end
            end
          end
        end
      end
    end
  end
end

if strcmpi(TextOut,'Yes') % make text output if asked
  cnt2 = 0; cnt3 = 0; cnt4 = 0; cnt5 = 0;
  t1 = 'ABCDEFG';
  for ii=1:ND-1
    for jj=ii+1:ND-1
      t2{ii}{jj} = [ t1(ii) t1(jj) ];
      for kk=jj+1:ND-1
        t3{ii}{jj}{kk} = [ t1(ii) t1(jj) t1(kk) ];
        for ll=kk+1:ND-1
          t4{ii}{jj}{kk}{ll} = [ t1(ii) t1(jj) t1(kk) t1(ll) ];
          for mm=ll+1:ND-1
              t5{ii}{jj}{kk}{ll}{mm} = [ t1(ii) t1(jj) t1(kk) t1(ll) t1(mm) ];
          end
        end
      end
    end
  end
  fprintf('\n  %d-way within-subject ANOVA tabular results:\n', ND-1);
  fprintf('_________________________________________________________________________________\n');
  fprintf('    Effect     SS         df          MS        F       p     p_GG   p_HF\n');
  fprintf('                                                              e_hat  etilde\n' );
  fprintf('_________________________________________________________________________________\n');
  for ii = 1:ND-1
    fprintf( '      %1s\t%12.2f\t%4d\t%10.2f\t%.2f\t%.4f\t%.4f\t%.4f\n', ...
      t1(ii), ANOVA_WS.AtoZ(ii,1:end-2) );
    fprintf( '    %1sxS\t%12.2f\t%4d\t%10.2f\t\t\t%.3f\t%.3f\n', ...
      t1(ii), ANOVA_WS.AStoZS(ii,:), ANOVA_WS.AtoZ(ii,end-1:end) );
  end
  for ii = 1:ND-2
    for jj = ii+1:ND-1
      cnt2 = cnt2 + 1;
      fprintf('     %2s\t%12.2f\t%4d\t%10.2f\t%.2f\t%.4f\t%.4f\t%.4f\n', ...
        t2{ii}{jj}, ANOVA_WS.AB(cnt2,1:end-2) );
      fprintf('   %2sxS\t%12.2f\t%4d\t%10.2f\t\t\t%.3f\t%.3f\n', ...
        t2{ii}{jj}, ANOVA_WS.ABS(cnt2,:), ANOVA_WS.AB(cnt2,end-1:end) );
    end
  end
  %
  % third order effects
  if ND-1 >= 3
    for ii = 1:ND-3
      for jj = ii+1:ND-2
        for kk = jj+1:ND-1
          cnt3 = cnt3 + 1;
          fprintf( '    %3s\t%12.2f\t%4d\t%10.2f\t%.2f\t%.4f\t%.4f\t%.4f\n', ...
            t3{ii}{jj}{kk}, ANOVA_WS.ABC(cnt3,1:end-2) );
          fprintf( '  %3sxS\t%12.2f\t%4d\t%10.2f\t\t\t%.3f\t%.3f\n', ...
            t3{ii}{jj}{kk}, ANOVA_WS.ABCS(cnt3,:), ANOVA_WS.ABC(cnt3,end-1:end) );
        end
      end
    end
  end
  %
  % fourth order effects
  if ND-1 >= 4
    for ii = 1:ND-4
      for jj = ii+1:ND-3
        for kk = jj+1:ND-2
          for ll = kk+1:ND-1
            cnt4 = cnt4 + 1;
            fprintf( '  %4s\t%12.2f\t%4d\t%10.2f\t%.2f\t%.4f\t%.4f\t%.4f\n', ...
              t4{ii}{jj}{kk}{ll}, ANOVA_WS.ABCD(cnt4,1:end-2) );
            fprintf( '%4sxS\t%12.2f\t%4d\t%10.2f\t\t\t%.3f\t%.3f\n', ...
              t4{ii}{jj}{kk}{ll}, ANOVA_WS.ABCDS(cnt4,:), ANOVA_WS.ABCD(cnt4,end-1:end) );
          end
        end
      end
    end
  end
  %
  % fifth order effects
  if ND-1 >= 5
    for ii = 1:ND-5
      for jj = ii+1:ND-4
        for kk = jj+1:ND-3
          for ll = kk+1:ND-2
            for mm = ll+1:ND-1
              cnt5 = cnt5 + 1;
              fprintf( ' %5s\t%12.2f\t%4d\t%10.2f\t%.2f\t%.4f\t%.4f\t%.4f\n', ...
                t5{ii}{jj}{kk}{ll}{mm}, ANOVA_WS.ABCDE(cnt5,1:end-2) );
              fprintf( '%5sxS\t%12.2f\t%4d\t%10.2f\t\t\t%.3f\t%.3f\n', ...
                t5{ii}{jj}{kk}{ll}{mm}, ANOVA_WS.ABCDES(cnt5,:), ANOVA_WS.ABCDE(cnt5,end-1:end) );
            end
          end
        end
      end
    end
  end
  fprintf('_________________________________________________________________________________\n');
end
%

end % end of main function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% functions for Orthogonal polynomials for the e-hat and e-tilde computation
function [ M, MList ] = GenOrthogComps( nTreats, NormFlag )
%
%     function [M,MList] = GenOrthogComps(nTreats,NormFlag)
%
% This will generate coefficients corresponding to one particular set of
% orthogonal copmarisons for a set of variables, given an input degrees of
% freedom for each variable.
%
% This is used for multiple comparisons following anova's as well as in the
% huynh-feldt corrections for repeated measures anovas
%
% Inputs:
%   df          - a vector of variable length denoting the degrees of freedom
%              (as in num treatments-1) for each factor. The length of dfs is
%              the number of factors
%   NormFlag    - set to 1 or 0 to determine wether or not you want to
%               normalize the output coefficients. normalization happens so that
%               the sum of the squares of each row of each output matrix sum to
%               1. Defaults to 1.
%
% Outputs:
%   Mout   -    this is a cell array of a matrix of coefficients
%               corresponding to the matrices M in the Huynh article from 1978
%               that correspond to each main effect and each interaction.
%
%               For example, if nTreats= [3], meaning one factor with 3
%               different levels, the ouput will be: M= [1   -1   0;
%                                                        1/2 1/2 -1]
%               which corresponds to the first comparison of u1-u2 and a second
%               orthogonal copmarison of 1/2*u1 + 1/2*u2 -u3
%
%               Main effects are presented in the first Mout, followed by
%               interactions, starting with 2 way interactions of the first two
%               factors, then 3 way interactions after all the 2-way
%               interactions, etc.
%
%   MList       This is a cell array of a text list of the effects for the
%               corresponding matrices in M. for example 'A' means main effect
%               for the first factor, 'AB' means a 2way interaction for the
%               first two factors, etc.
%
% To get the matrices needed for the Huynh-Feldt correction, only
% input the nTreats for all of the within subject factors in your
% design.
%
% Note this outputs one (relatively easy) way to get orthogonal copmarisons, but
% be aware that tere are infinite different ways to come up with a set of
% orthogonal copmarisons.
%
% For exapmle, running GenOrthogComps([3 3]) give identical outputs to the M
% matrices of table 1 in Huynh H. (1978) "Some approximate tests for repeated
% measurement designs", Psychometrika
%
% written on 090131 by Matthew Nelson

if nargin<2 || isempty(NormFlag), NormFlag=1; end
% used for MList. If you have more than 26 factors this will result in an error,
% but if your expeirmental design has more than 26 factors, you may need to stop
% and rethink a whole number of things
alphabet='ABCDEFGHIJKLMNOPQRSTUVWXYZ';

dfs=nTreats-1;
nFacs=length(nTreats);
nCombs=prod(nTreats);

%first do comps for main effects
M=cell(1,nFacs);
for iFac=1:nFacs
  M{iFac} = repmat(0,dfs(iFac),nCombs);
  if iFac==nFacs;
    nLowCombs=1;
  else
    nLowCombs=prod(nTreats(iFac+1:end));
  end
  %
  if iFac==1;
    nUpCombs=1;
  else
    nUpCombs=prod(nTreats(1:iFac-1));
  end
  %
  for idf=1:dfs(iFac)
    tmpComps=repmat( 0,1,nLowCombs*nTreats(iFac) );
    tmpComps( 1:nLowCombs*idf )=1/idf;
    tmpComps( nLowCombs*idf+1:nLowCombs*(idf+1) )=-1;
    tmpComps=repmat(tmpComps,1,nUpCombs);
    %
    M{iFac}(idf, : )=tmpComps;
    MList{iFac}=alphabet(iFac);
  end
end

% Now do comps for any interactions as products of the main effect comps ...
% I believe this requires recursion
if nFacs>1
  % start with 2 way interactions and go up to nFacs-way interactions
  for curnFacs=2:nFacs
    for initFac=1:nFacs-(curnFacs-1)
      [ M MList ] = RLoop( curnFacs, initFac, repmat( 1, 1, nCombs ), ...
        length(M)+1, '', M, MList, dfs, alphabet );
    end
  end
end

% optionally normalize coefficients
if NormFlag
  for im=1:length(M)
    for ir=1:size(M{im},1)
      M{im}(ir,:)=M{im}(ir,:)/ sqrt((sum(M{im}(ir,:).^2)));
    end
  end
end
end % end of function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ M MList ] = RLoop( FacRem, curFac, curRow, curMNum, ...
  nextList, M, MList, dfs, alphabet )
% given the initial inputs, and the initial curFac, this should calc all the
% downstream M vals for that initial curFac...

FacRem=FacRem-1;
for idf=1:dfs(curFac)
  if FacRem==0
    if curMNum>length(M);
      M{curMNum}=[];
      MList{curMNum}=[nextList alphabet(curFac)];
    end
    M{curMNum}(end+1,:)=curRow.*M{curFac}(idf,:);
  else
    nextRow=curRow.*M{curFac}(idf,:);
    nextList(end+1)=alphabet(curFac);
    baseFacNum=curFac+1;     %this is needed for the numbering of the outputs...
    for iNextFac=curFac+1:length(dfs)-FacRem+1
      [ M MList ] = RLoop( FacRem, iNextFac, nextRow, ...
        curMNum + ( iNextFac - baseFacNum ), nextList, M, MList, dfs, alphabet );
    end
  end
end
end % end of function
%
%