% README tt 20230313 for manuscript/preprint/article
% Huang, Zhou, Tzvetanov, Influences of local and global context on local
% orientation perception, 2023, doi 10.22541/au.167671447.78006473/v1
%
% ALL FUNCTIONS AND SCRIPTS IN THIS DIRECTORY FOLLOW THE STATEMENT BELOW
%
## This program is free software; you can redistribute it and/or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation; either version 3 of the License, or (at your option) any later
## version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, see <http://www.gnu.org/licenses/>.
%
%
% Scripts and functions for analysis of the data from EXP1 (replication)
% and EXP5 (main), to backup the claims in the manuscript.
% this directory contains the scripts for:
% 1. rt_plots_analysis_3.m : making the three figures for experimental
%   results in the manuscript of biases, thresholds, response times
% 2. power_effectsize_exp5_exp1_1.m : running the within subject ANOVAs
%   and power analysis and print it out on the command window
% 3. power_effectsize_exp5_2.m : making the simulation for testing at
%   what change of flanks's bias curve wrt 90deg condition one should be
%   able to obtain significant interaction (see manuscript for explanations)
% 4. rt_indivblock_stats_exp5_1.m : runs the 58 individual blocks anovan()
%   tests following the RTs analysis in the manuscript, and printing out
%   the binomial probability to obtain the resulting amount of significant
%   interaction effects.
%
% Other scripts in the directory, necessary for the analysis :
% nANOVA_WS.m -> my within subject anova analysis function
% ncfcdf_ab.m -> noncentral F-cumulative density function to compute power,
%   !!! IMPORTANT: this function was taken from the current (february-march
%   2023) online GNU OCTAVE statistic package, because it is not present in the
%   gnu octave versions 6.4 and 7.3 i use on Lubuntu (22.04) or Win10.
% EffectsizePrintOut.m -> my function to compute power and effect sizes and
%   make print out (if demanded) on the command window
% populationprevalence_cw.m -> small function to compute the population
%   prevalence as presented in the papers of Ince et al. (see function help)
%
% DATA FILES :
% (i) subjects 3 letter code is taken out
% (ii) all data files include preprocessed data (see relevant scripts for what
%   is used) and raw data (usually in a cell array, see explanations below)
% 1. localglobal_hjf_exp5_data_analysis.mat : the bias and threshold data as
%   presented in all manuscripts (preprints); it contains the RTs data that were
%   presented in preprint v1 (10.22541/au.167671447.78006473/v1)
% 2. localglobal_hjf_exp5_data_analysis_onlyRTs.mat : the RTs data as analysed
%   in v2 of the preprint; the main change was to (1) first prune RTs and then
%   (2) adjust for differences in left-right keys response times; the outcome is
%   very similar (especially on plot), though the exact statistic slightly
%   changed.
% 3. localglobal_hjf_exp1_OneLapseInBlock_data_analysis.mat: main EXP1 data, of
%   a sine-carrier seen through an elongated Gaussian envelope.
% 4. two .xlsx files of the data extracted from Dakin et al., 1999, Vision
%   Research article (in case someone is interested to compare to our results)
%
% structures in the .mat data files:
% in localglobal_hjf_exp5_data_analysis.mat and [...]_onlyRTs.mat:
%  exp5 =
%      subject: 1x7 cell
%      files: 1x7 cell
%      condition: 1x7 cell
%      Envelope_or: 1x8 matrix
%      Flank_or: 1x12 matrix
%      Fits: 1x7 cell  -> Fits{indexsubject}{indexfile} raw data (X,Y,N,rtraw) and
%       fits (psych.function results) for all conditions (rows in X,Y) in a file
%      Nbad_rt_per_file_4: 10x2x7 matrix -> nb RTs pruned in block at robSTD=4
%      Block_left_right_seL_seR: 10x4x7 matrix
%      Block_Mean_median_se_RTall: 10x4x7 matrix
%      Block_Mean_median_se_RTthr4: 10x4x7 matrix
%      filenumber: 8x12x7 matrix
%      rt_4thr: 8x12x7 matrix -> computed RT threshold at 4xrobSTD
%      Biases: 8x12x7 matrix -> adjusted biases for [env,fl,subject]
%      Thresholds: 8x12x7 matrix -> thresholds for [env,fl,subject]
%      rt_raw_mean: 8x12x7 matrix
%      rt_all_mean: 8x12x7 matrix
%      rt_4thr_mean: 8x12x7 matrix -> pruned+leftright adjusted mean RTs for
%       each [env,fl,subject], needs block adjustment to correspond to preprint
%      Ngood_rt_4: 8x12x7 matrix -> nb RTs remaining for each [env,fl,subject]
%      rt_raw: 8x12x7 cell
%      rt_leftrightadjusted: 8x12x7 cell
%      y_all: 8x12x7 cell
%      x_all: 8x12x7 cell
%      rtblockmean: 8x12x7 matrix
%      rtblockmean4thr: 8x12x7 matrix
%      aov: 1x1 scalar struct
%
% in localglobal_hjf_exp1_OneLapseInBlock_data_analysis.mat: same as for exp5
%  exp1 =
%      subject: 11x1 cell
%      Envelope_or: 1x18 matrix
%      Fits: 1x11 cell
%      Nbad_rt_per_file_4: 10x2x11 matrix
%      Block_left_right_seL_seR: 10x4x11 matrix
%      Block_Mean_median_se_RTall: 10x4x11 matrix
%      Block_Mean_median_se_RTthr4: 10x4x11 matrix
%      files: 11x1 cell
%      filenumber: 18x11 matrix
%      rt_4thr: 18x11 matrix
%      Biases: 18x11 matrix
%      Thresholds: 18x11 matrix
%      Lapse: 18x11 matrix
%      rt_raw_mean: 18x11 matrix
%      rt_all_mean: 18x11 matrix
%      rt_4thr_mean: 18x11 matrix
%      Ngood_rt_4: 18x11 matrix
%      rt_raw: 18x11 cell
%      rt_leftrightadjusted: 18x11 cell
%      y_all: 18x11 cell
%      x_all: 18x11 cell
%      rtblockmean: 18x11 matrix
%      rtblockmean4thr: 18x11 matrix
%      
