## This program is free software; you can redistribute it and/or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation; either version 3 of the License, or (at your option) any later
## version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, see <http://www.gnu.org/licenses/>.
##
## Tzvetomir Tzvetanov, 2023/03/09
##

% make individual subjects analysis for manuscript
% each block i sindividually analysed with between subject anova to test
% each of the 58 blocks for significant interaction effect

clear all;

pkg load io statistics;
addpath ScriptsForAnalysis;

load localglobal_hjf_exp5_data_analysis.mat;

fl = exp5.Flank_or;

nprune = 4; percprune = 0.05; robSTD = 4.0; nbrtpruned = 0;
for isubj=1:7
  nbl = numel(exp5.Fits{isubj});
  fprintf('  --> Subject %d\n', isubj);
  for ib=1:nbl
    
    fprintf('Block %2d, [env,fl]=[%3d,%3d],', ib,exp5.Fits{isubj}{ib}.envfl(1,:));
    
    rtb=[];
    rtb = log10( exp5.Fits{isubj}{ib}.rtraw' );
    y = exp5.Fits{isubj}{ib}.Y';
    
    % this makes "robSTD" robust STD pruning
    rl = median(rtb(:))+robSTD*1.4826*mad(rtb(:),1);
    y(rtb>rl) = nan; rtb(rtb>rl) = nan; nbrtpruned = 100*sum(isnan(rtb(:)))/numel(rtb);
    
    % now adjust for Left Right differences, and save th eLR values
    rtm = nanmean(rtb(:));
    rtb_lr_m = [nanmean(rtb(y==0)(:)),nanmean(rtb(y==1)(:))];
    rtb_lr_se = [nanstd(rtb(y==0)(:))/sqrt(nansum(y(:)==0)),nanstd(rtb(y==1)(:))/sqrt(nansum(y(:)==1))];
    zlr = diff(rtb_lr_m) ./ sqrt(0.5*sum(rtb_lr_se.^2));
    
    indivblock(isubj).LeftRight_means_SEM_Z(ib,:) = [rtb_lr_m, rtb_lr_se, zlr];
    indivblock(isubj).nbrtpruned(ib) = nbrtpruned;
    
    % raw RT analysis with kruskal wallis test
    indivblock(isubj).p_raw_lradj(ib,1) = kruskalwallis( rtb,[],'off' );
    rtb2 = rtb;
    rtb2(y==0) = rtb2(y==0) - rtb_lr_m(1) + rtm;
    rtb2(y==1) = rtb2(y==1) - rtb_lr_m(2) + rtm;
    indivblock(isubj).p_raw_lradj(ib,2) = kruskalwallis( rtb2,[],'off' );
    
    % anova
    ne = numel(unique( exp5.Fits{isubj}{ib}.envfl(:,1) ));
    nfl = numel(unique( exp5.Fits{isubj}{ib}.envfl(:,2) ));
    % test anovan() in gnu octave
    rta=[]; grps=[]; n=rows(rtb);for ii=1:ne for jj=1:nfl rta=[rta;rtb(:,(ii-1)*nfl+jj)]; grps=[grps;ones(n,1)*[ii,jj]]; endfor; endfor;grps(isnan(rta),:)=[];rta(isnan(rta))=[];
    [indivblock(isubj).anovan_raw(ib,1:3),indivblock(isubj).anovan_raw_F(ib,1:3),indivblock(isubj).anovan_raw_df(ib,1:3)] = ...
      anovan( rta,grps, 'model', 'interaction' );
    indivblock(isubj).panovan_GL(ib,1) = indivblock(isubj).anovan_raw(ib,3);
    rta=[]; grps=[]; n=rows(rtb2);for ii=1:ne for jj=1:nfl rta=[rta;rtb2(:,(ii-1)*nfl+jj)]; grps=[grps;ones(n,1)*[ii,jj]]; endfor; endfor;grps(isnan(rta),:)=[];rta(isnan(rta))=[];
    [indivblock(isubj).anovan_lradj(ib,1:3),indivblock(isubj).anovan_lradj_F(ib,1:3),indivblock(isubj).anovan_lradj_df(ib,1:3)] = ...
      anovan( rta,grps, 'model', 'interaction' );
    indivblock(isubj).panovan_GL(ib,2) = indivblock(isubj).anovan_lradj(ib,3);
    
    fprintf('  Zlr %6.2f, p_kw %.4f %.4f, Pruned %4.1f; AOV G %.3f L %.3f GL %.3f; lradj G %.3f L %.3f GL %.3f\n', ...
      zlr,indivblock(isubj).p_raw_lradj(ib,:),nbrtpruned,...
      indivblock(isubj).anovan_raw(ib,:), ...
      indivblock(isubj).anovan_lradj(ib,:) );
  endfor
  indivsubj(isubj,1) = sum(indivblock(isubj).panovan_GL(:,2)<0.10)>0;
  indivsubj(isubj,2) = sum(indivblock(isubj).panovan_GL(:,2)<0.05)>0;
  indivsubj(isubj,3) = sum(indivblock(isubj).panovan_GL(:,2)<0.01)>0;
  fprintf('      Nb sign. KW tests p<0.10 %d %d, aovn GL p<0.10 %d %d / %d\n', ...
    sum(indivblock(isubj).p_raw_lradj<0.10), sum(indivblock(isubj).panovan_GL<0.10), nbl);
  fprintf('      Nb sign. KW tests p<0.05 %d %d, aovn GL p<0.05 %d %d / %d\n', ...
    sum(indivblock(isubj).p_raw_lradj<0.05), sum(indivblock(isubj).panovan_GL<0.05), nbl);
endfor

% now check how many blocks have interaction p<0.05
ps=[]; for ii=1:7 ps=[ps;indivblock(ii).panovan_GL]; endfor
p5 = binocdf(sum(ps<0.05)-1,rows(ps),0.05,'upper'); % tt 20230311 small change
fprintf( '\n\n  rawRTs = RTs with only pruning at 4*robSTD\n' );
fprintf( '  LRadjRTs = RTs with additionally adjustment of Left-Right response asymetries\n' );
fprintf( '    rawRTs    LRadjRTs\np=  %.5f  %.5f\n', p5 );

% here check how many subject have at least one block with interaction<alpha and
% compute population prevalence
r(1) = populationprevalence_cw( sum(indivsubj(:,1)), 7, 0.10, 0.96 );
r(2) = populationprevalence_cw( sum(indivsubj(:,2)), 7, 0.05, 0.96 );
r(3) = populationprevalence_cw( sum(indivsubj(:,3)), 7, 0.01, 0.96 );
fprintf( '\n\n  Population prevalence estimate: alpha=%.2f, map=%.3f, hpdi(%2d)=[%.2f,%.2f]\n', 0.1, r(1).map, r(1).hpdiprob, r(1).hpdi );
fprintf( '  Population prevalence estimate: alpha=%.2f, map=%.3f, hpdi(%2d)=[%.2f,%.2f]\n', 0.05, r(2).map, r(2).hpdiprob, r(2).hpdi );
fprintf( '  Population prevalence estimate: alpha=%.2f, map=%.3f, hpdi(%2d)=[%.2f,%.2f]\n', 0.01, r(3).map, r(3).hpdiprob, r(3).hpdi );
